@@ -9,7 +9,8 @@ import os
 import platform
 import sys
 
-from typing import Any, Iterable, Tuple
+from typing import Any
+from collections.abc import Iterable
 
 # debug_info() at the bottom wants to show all the globals, but not imports.
 # Grab the global names here to know which names to not show. Nothing defined
@@ -34,7 +35,10 @@ PYPY = (platform.python_implementation() == "PyPy")
 PYVERSION = sys.version_info + (int(platform.python_version()[-1] == "+"),)
 
 if PYPY:
+    # Minimum now is 7.3.16
     PYPYVERSION = sys.pypy_version_info         # type: ignore[attr-defined]
+else:
+    PYPYVERSION = (0,)
 
 # Python behavior.
 class PYBEHAVIOR:
@@ -51,38 +55,15 @@ class PYBEHAVIOR:
     # across versions.
     if pep626:
         optimize_if_not_debug = 1
-    elif PYPY:
-        if PYVERSION >= (3, 9):
-            optimize_if_not_debug = 2
-        else:
-            optimize_if_not_debug = 3
     else:
         optimize_if_not_debug = 2
 
     # 3.7 changed how functions with only docstrings are numbered.
     docstring_only_function = (not PYPY) and (PYVERSION <= (3, 10))
 
-    # When a break/continue/return statement in a try block jumps to a finally
-    # block, does the finally jump back to the break/continue/return (pre-3.10)
-    # to do the work?
-    finally_jumps_back = (PYVERSION < (3, 10))
-
-    # CPython 3.11 now jumps to the decorator line again while executing
-    # the decorator.
-    trace_decorator_line_again = (CPYTHON and PYVERSION > (3, 11, 0, "alpha", 3, 0))
-
-    # CPython 3.9a1 made sys.argv[0] and other reported files absolute paths.
-    report_absolute_files = (
-        (CPYTHON or (PYPY and PYPYVERSION >= (7, 3, 10)))
-        and PYVERSION >= (3, 9)
-    )
-
     # Lines after break/continue/return/raise are no longer compiled into the
     # bytecode.  They used to be marked as missing, now they aren't executable.
-    omit_after_jump = (
-        pep626
-        or (PYPY and PYVERSION >= (3, 9) and PYPYVERSION >= (7, 3, 12))
-    )
+    omit_after_jump = pep626 or PYPY
 
     # PyPy has always omitted statements after return.
     omit_after_return = omit_after_jump or PYPY
@@ -100,27 +81,24 @@ class PYBEHAVIOR:
     # When leaving a with-block, do we visit the with-line again for the exit?
     exit_through_with = (PYVERSION >= (3, 10, 0, "beta"))
 
+    # When leaving a with-block, do we visit the with-line exactly,
+    # or the inner-most context manager?
+    exit_with_through_ctxmgr = (PYVERSION >= (3, 12))
+
     # Match-case construct.
     match_case = (PYVERSION >= (3, 10))
 
     # Some words are keywords in some places, identifiers in other places.
     soft_keywords = (PYVERSION >= (3, 10))
 
-    # Modules start with a line numbered zero. This means empty modules have
-    # only a 0-number line, which is ignored, giving a truly empty module.
-    empty_is_empty = (PYVERSION >= (3, 11, 0, "beta", 4))
-
-    # Are comprehensions inlined (new) or compiled as called functions (old)?
-    # Changed in https://github.com/python/cpython/pull/101441
-    comprehensions_are_functions = (PYVERSION <= (3, 12, 0, "alpha", 7, 0))
-
     # PEP669 Low Impact Monitoring: https://peps.python.org/pep-0669/
     pep669 = bool(getattr(sys, "monitoring", None))
 
     # Where does frame.f_lasti point when yielding from a generator?
-    # It used to point at the YIELD, now it points at the RESUME.
+    # It used to point at the YIELD, in 3.13 it points at the RESUME,
+    # then it went back to the YIELD.
     # https://github.com/python/cpython/issues/113728
-    lasti_is_yield = (PYVERSION < (3, 13))
+    lasti_is_yield = (PYVERSION[:2] != (3, 13))
 
 
 # Coverage.py specifics, about testing scenarios. See tests/testenv.py also.
@@ -134,7 +112,7 @@ METACOV = os.getenv("COVERAGE_COVERAGE") is not None
 TESTING = os.getenv("COVERAGE_TESTING") == "True"
 
 
-def debug_info() -> Iterable[Tuple[str, Any]]:
+def debug_info() -> Iterable[tuple[str, Any]]:
     """Return a list of (name, value) pairs for printing debug information."""
     info = [
         (name, value) for name, value in globals().items()
