@@ -323,6 +323,19 @@ class WithTest(CoverageTest):
         expected = "line 3 didn't jump to the function exit"
         assert self.get_missing_arc_description(cov, 3, -2) == expected
 
+    def test_leaving_module(self) -> None:
+        cov = self.check_coverage("""\
+            print(a := 1)
+            if a == 1:
+                print(3)
+            """,
+            branchz="2. 23",
+            branchz_missing="2.",
+        )
+        assert self.stdout() == "1\n3\n"
+        expected = "line 2 didn't exit the module because the condition on line 2 was always true"
+        assert self.get_missing_arc_description(cov, 2, -1) == expected
+
     def test_with_with_lambda(self) -> None:
         self.check_coverage("""\
             from contextlib import nullcontext
@@ -334,6 +347,41 @@ class WithTest(CoverageTest):
             branchz_missing="",
         )
 
+    def test_multiline_with(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/1880
+        self.check_coverage("""\
+            import contextlib, itertools
+            nums = itertools.count()
+            with (
+                contextlib.nullcontext() as x,
+            ):
+                while next(nums) < 6:
+                    y = 7
+            z = 8
+            """,
+            branchz="67 68",
+            branchz_missing="",
+        )
+
+
+    def test_multi_multiline_with(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/1880
+        self.check_coverage("""\
+            import contextlib, itertools
+            nums = itertools.count()
+            with (
+                contextlib.nullcontext() as x,
+                contextlib.nullcontext() as y,
+                contextlib.nullcontext() as z,
+            ):
+                while next(nums) < 8:
+                    y = 9
+            z = 10
+            """,
+            branchz="89 8A",
+            branchz_missing="",
+        )
+
 
 class LoopArcTest(CoverageTest):
     """Arc-measuring tests involving loops."""
